#include <stdio.h>
#include <unistd.h>
#include <linux/perf_event.h>
#include <sys/syscall.h>

/* Level 2 for you! */

static struct perf_event_attr info;
int fd;

static inline int
sys_perf_event_open(struct perf_event_attr *attr,
                      pid_t pid, int cpu, int group_fd,
                      unsigned long flags)
{
        attr->size = sizeof(*attr);
        return syscall(__NR_perf_event_open, attr, pid, cpu,
                       group_fd, flags);
}


int setup()
{
	/* Set up the event structure. */

	info.type = 0;

	/* ... Event type 0 (hardware), event id 0 (CPU cycles). */
	info.config = 0; 

	info.size = sizeof(info);

	info.sample_period = 0;

	info.sample_type = 0;

	/* ... extra counter information on read? no thanks */
	info.read_format = 0;

	/* ... options -- many of which are meaningless for CPU
	 * cycle counting. */
	info.disabled = info.inherit = info.pinned = info.exclusive
		= info.exclude_user = info.exclude_kernel = info.exclude_hv
		= info.exclude_idle = info.mmap = info.comm
		= 0;

	/* ... no extra wake-up required */
	info.wakeup_events = 0;

	/* ... pid = -1 (all processes), cpu = 0 (this CPU),
	 *     group_fd = -1 (group leader), flags = 0 (unused) */
	fd = sys_perf_event_open(&info, -1, 0, -1, 0);

	return fd;
};

int sample(uint64_t *val)
{
	if (read(fd, val, sizeof(*val)) != sizeof(*val))
		return -1;
	return 0;
}

int main()
{
	uint64_t val;
	int result;

	if((result = setup()) < 0) {
		printf("Setup failed: %d\n", result);
		perror("Perf");
		return 1;
	}

	sample(&val);
	printf("Val 1: %llx\n", val);
	sleep(5);
	sample(&val);
	printf("Val 2: %llx\n", val);

	return 0;
}
